@@ -61,9 +61,9 @@ struct TP_QT_NO_EXPORT ContactManager::Private
     WeakPtr<Connection> connection;
     ContactManager::Roster *roster;
 
-    QMap<uint, WeakPtr<Contact> > contacts;
+    QHash<uint, WeakPtr<Contact> > contacts;
 
-    QMap<Feature, bool> tracking;
+    QHash<Feature, bool> tracking;
     Features supportedFeatures;
 
     // avatar
@@ -252,7 +252,8 @@ Features ContactManager::supportedFeatures() const
             << Contact::FeatureLocation
             << Contact::FeatureInfo
             << Contact::FeatureRosterGroups
-            << Contact::FeatureAddresses;
+            << Contact::FeatureAddresses
+            << Contact::FeatureClientTypes;
         QStringList interfaces = connection()->lowlevel()->contactAttributeInterfaces();
         foreach (const Feature &feature, allFeatures) {
             if (interfaces.contains(featureToInterface(feature))) {
@@ -1084,14 +1085,14 @@ PendingContacts *ContactManager::contactsForIdentifiers(const QStringList &ident
 }
 
 /**
- * Request contacts and enable their \a features using a given field in their vCards.
+ * Request contacts and enable their \a features using a given field in their vcards.
  *
  * This method requires Connection::FeatureCore to be ready.
  *
- * \param vcardField The vCard field of the addresses we are requesting.
+ * \param vcardField The vcard field of the addresses we are requesting.
  *                   Supported fields can be found in ProtocolInfo::addressableVCardFields().
  * \param vcardAddresses The addresses to get contacts for. The address types must match
- *                       the given vCard field.
+ *                       the given vcard field.
  * \param features The Contact features to enable.
  * \return A PendingContacts, which will emit PendingContacts::finished
  *         when the contacts are retrieved or an error occurred.
@@ -1340,17 +1341,27 @@ void ContactManager::onAvatarRetrieved(uint handle, const QString &token,
         debug() << "Filename:" << avatarFileName;
         debug() << "MimeType:" << mimeType;
 
-        QTemporaryFile mimeTypeFile(mimeTypeFileName);
-        mimeTypeFile.open();
-        mimeTypeFile.write(mimeType.toLatin1());
-        mimeTypeFile.setAutoRemove(false);
-        mimeTypeFile.rename(mimeTypeFileName);
+        if (!QFile::exists(mimeTypeFileName)) {
+            QTemporaryFile mimeTypeFile(mimeTypeFileName);
+            if (mimeTypeFile.open()) {
+                mimeTypeFile.write(mimeType.toLatin1());
+                mimeTypeFile.setAutoRemove(false);
+                if (!mimeTypeFile.rename(mimeTypeFileName)) {
+                    mimeTypeFile.remove();
+                }
+            }
+        }
 
-        QTemporaryFile avatarFile(avatarFileName);
-        avatarFile.open();
-        avatarFile.write(data);
-        avatarFile.setAutoRemove(false);
-        avatarFile.rename(avatarFileName);
+        if (!QFile::exists(avatarFileName)) {
+            QTemporaryFile avatarFile(avatarFileName);
+            if (avatarFile.open()) {
+                avatarFile.write(data);
+                avatarFile.setAutoRemove(false);
+                if (!avatarFile.rename(avatarFileName)) {
+                    avatarFile.remove();
+                }
+            }
+        }
     }
 
     ContactPtr contact = lookupContactByHandle(handle);
@@ -1408,6 +1419,17 @@ void ContactManager::onContactInfoChanged(uint handle, const Tp::ContactInfoFiel
     }
 }
 
+void ContactManager::onClientTypesUpdated(uint handle, const QStringList &clientTypes)
+{
+    debug() << "Got ClientTypesUpdated for contact with handle" << handle;
+
+    ContactPtr contact = lookupContactByHandle(handle);
+
+    if (contact) {
+        contact->receiveClientTypes(clientTypes);
+    }
+}
+
 void ContactManager::doRefreshInfo()
 {
     PendingRefreshContactInfo *op = mPriv->refreshInfoOp;
@@ -1472,6 +1494,8 @@ QString ContactManager::featureToInterface(const Feature &feature)
         return TP_QT_IFACE_CONNECTION_INTERFACE_CONTACT_GROUPS;
     } else if (feature == Contact::FeatureAddresses) {
         return TP_QT_IFACE_CONNECTION_INTERFACE_ADDRESSING;
+    } else if (feature == Contact::FeatureClientTypes) {
+        return TP_QT_IFACE_CONNECTION_INTERFACE_CLIENT_TYPES;
     } else {
         warning() << "ContactManager doesn't know which interface corresponds to feature"
             << feature;
@@ -1536,6 +1560,13 @@ void ContactManager::ensureTracking(const Feature &feature)
         connect(simplePresenceInterface,
                 SIGNAL(PresencesChanged(Tp::SimpleContactPresences)),
                 SLOT(onPresencesChanged(Tp::SimpleContactPresences)));
+    } else if (feature == Contact::FeatureClientTypes) {
+        Client::ConnectionInterfaceClientTypesInterface *clientTypesInterface =
+            conn->interface<Client::ConnectionInterfaceClientTypesInterface>();
+
+        connect(clientTypesInterface,
+                SIGNAL(ClientTypesUpdated(uint,QStringList)),
+                SLOT(onClientTypesUpdated(uint,QStringList)));
     } else if (feature == Contact::FeatureRosterGroups || feature == Contact::FeatureAddresses) {
         // nothing to do here, but we don't want to warn
         ;
